@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,12 +21,23 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * $Log: httpform.h,v $
+ * Revision 1.18  2005/11/30 12:47:37  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.17  2003/03/24 04:30:41  robertj
+ * Added function to set and get strings from PConfig in correct format for
+ *   use with HTTP form array contsructs.
+ *
+ * Revision 1.16  2002/11/06 22:47:24  robertj
+ * Fixed header comment (copyright etc)
+ *
+ * Revision 1.15  2002/09/16 01:08:59  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
  * Revision 1.14  2001/10/10 08:07:48  robertj
  * Fixed large memory leak of strings when doing POST to a form.
  *
@@ -74,7 +85,7 @@
 #ifndef _PHTTPFORM
 #define _PHTTPFORM
 
-#ifdef __GNUC__
+#ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
@@ -410,6 +421,15 @@ class PHTTPFieldArray : public PHTTPCompositeField
     virtual PINDEX GetSize() const;
     void SetSize(PINDEX newSize);
 
+    PStringArray GetStrings(
+      PConfig & cfg   ///< Config file to get strings from
+    );
+
+    void SetStrings(
+      PConfig & cfg,   ///< Config file to Set strings to
+      const PStringArray & values ///< Strings to set
+    );
+
   protected:
     void AddBlankField();
     void AddArrayControlBox(PHTML & html, PINDEX fld) const;
@@ -442,7 +462,7 @@ class PHTTPStringField : public PHTTPField
     virtual PHTTPField * NewField() const;
 
     virtual void GetHTMLTag(
-      PHTML & html    // HTML to receive the field info.
+      PHTML & html    ///< HTML to receive the field info.
     ) const;
 
     virtual PString GetValue(BOOL dflt = FALSE) const;
@@ -480,7 +500,7 @@ class PHTTPPasswordField : public PHTTPStringField
     virtual PHTTPField * NewField() const;
 
     virtual void GetHTMLTag(
-      PHTML & html    // HTML to receive the field info.
+      PHTML & html    ///< HTML to receive the field info.
     ) const;
 
     virtual PString GetValue(BOOL dflt = FALSE) const;
@@ -516,7 +536,7 @@ class PHTTPIntegerField : public PHTTPField
     virtual PHTTPField * NewField() const;
 
     virtual void GetHTMLTag(
-      PHTML & html    // HTML to receive the field info.
+      PHTML & html    ///< HTML to receive the field info.
     ) const;
 
     virtual PString GetValue(BOOL dflt = FALSE) const;
@@ -526,10 +546,10 @@ class PHTTPIntegerField : public PHTTPField
     );
 
     virtual void LoadFromConfig(
-      PConfig & cfg   // Configuration for value transfer.
+      PConfig & cfg   ///< Configuration for value transfer.
     );
     virtual void SaveToConfig(
-      PConfig & cfg   // Configuration for value transfer.
+      PConfig & cfg   ///< Configuration for value transfer.
     ) const;
 
     virtual BOOL Validated(
@@ -564,7 +584,7 @@ class PHTTPBooleanField : public PHTTPField
     virtual PHTTPField * NewField() const;
 
     virtual void GetHTMLTag(
-      PHTML & html    // HTML to receive the field info.
+      PHTML & html    ///< HTML to receive the field info.
     ) const;
 
     virtual PString GetHTMLInput(
@@ -578,10 +598,10 @@ class PHTTPBooleanField : public PHTTPField
     );
 
     virtual void LoadFromConfig(
-      PConfig & cfg   // Configuration for value transfer.
+      PConfig & cfg   ///< Configuration for value transfer.
     );
     virtual void SaveToConfig(
-      PConfig & cfg   // Configuration for value transfer.
+      PConfig & cfg   ///< Configuration for value transfer.
     ) const;
 
 
@@ -658,7 +678,7 @@ class PHTTPRadioField : public PHTTPField
     virtual PHTTPField * NewField() const;
 
     virtual void GetHTMLTag(
-      PHTML & html    // HTML to receive the field info.
+      PHTML & html    ///< HTML to receive the field info.
     ) const;
 
     virtual PString GetHTMLInput(
@@ -716,7 +736,7 @@ class PHTTPSelectField : public PHTTPField
     virtual PHTTPField * NewField() const;
 
     virtual void GetHTMLTag(
-      PHTML & html    // HTML to receive the field info.
+      PHTML & html    ///< HTML to receive the field info.
     ) const;
 
     virtual PString GetValue(BOOL dflt = FALSE) const;
@@ -761,13 +781,13 @@ class PHTTPForm : public PHTTPString
 
 
     virtual void OnLoadedText(
-      PHTTPRequest & request,    // Information on this request.
-      PString & text             // Data used in reply.
+      PHTTPRequest & request,    ///< Information on this request.
+      PString & text             ///< Data used in reply.
     );
     virtual BOOL Post(
-      PHTTPRequest & request,       // Information on this request.
-      const PStringToString & data, // Variables in the POST data.
-      PHTML & replyMessage          // Reply message for post.
+      PHTTPRequest & request,       ///< Information on this request.
+      const PStringToString & data, ///< Variables in the POST data.
+      PHTML & replyMessage          ///< Reply message for post.
     );
 
 
@@ -830,13 +850,13 @@ class PHTTPConfig : public PHTTPForm
     );
 
     virtual void OnLoadedText(
-      PHTTPRequest & request,    // Information on this request.
-      PString & text             // Data used in reply.
+      PHTTPRequest & request,    ///< Information on this request.
+      PString & text             ///< Data used in reply.
     );
     virtual BOOL Post(
-      PHTTPRequest & request,       // Information on this request.
-      const PStringToString & data, // Variables in the POST data.
-      PHTML & replyMessage          // Reply message for post.
+      PHTTPRequest & request,       ///< Information on this request.
+      const PStringToString & data, ///< Variables in the POST data.
+      PHTML & replyMessage          ///< Reply message for post.
     );
 
 
@@ -852,7 +872,7 @@ class PHTTPConfig : public PHTTPForm
     const PString & GetConfigSection() const { return section; }
 
     void SetConfigSection(
-      const PString & sect   // New section for the config page.
+      const PString & sect   ///< New section for the config page.
     ) { section = sect; }
     // Set the configuration file section.
 
@@ -861,17 +881,17 @@ class PHTTPConfig : public PHTTPForm
        and error generated if the section already exists.
      */
     PHTTPField * AddSectionField(
-      PHTTPField * sectionFld,     // Field to set as the section name
-      const char * prefix = NULL,  // String to attach before the field value
-      const char * suffix = NULL   // String to attach after the field value
+      PHTTPField * sectionFld,     ///< Field to set as the section name
+      const char * prefix = NULL,  ///< String to attach before the field value
+      const char * suffix = NULL   ///< String to attach after the field value
     );
 
     /** Add fields to the HTTP form for adding a new key to the config file
        section.
      */
     void AddNewKeyFields(
-      PHTTPField * keyFld,  // Field for the key to be added.
-      PHTTPField * valFld   // Field for the value of the key yto be added.
+      PHTTPField * keyFld,  ///< Field for the key to be added.
+      PHTTPField * valFld   ///< Field for the value of the key yto be added.
     );
 
 
@@ -907,13 +927,13 @@ class PHTTPConfigSectionList : public PHTTPString
     );
 
     virtual void OnLoadedText(
-      PHTTPRequest & request,    // Information on this request.
-      PString & text             // Data used in reply.
+      PHTTPRequest & request,    ///< Information on this request.
+      PString & text             ///< Data used in reply.
     );
     virtual BOOL Post(
-      PHTTPRequest & request,       // Information on this request.
-      const PStringToString & data, // Variables in the POST data.
-      PHTML & replyMessage          // Reply message for post.
+      PHTTPRequest & request,       ///< Information on this request.
+      const PStringToString & data, ///< Variables in the POST data.
+      PHTML & replyMessage          ///< Reply message for post.
     );
 
   protected:
