@@ -27,19 +27,64 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: sockets.cxx,v $
- * Revision 1.177.2.4  2005/07/17 10:03:22  csoutheren
- * Backported fix from Atlas
+ * Revision 1.197.2.3  2007/01/03 22:37:37  dsandras
+ * Backports from HEAD.
  *
- * Revision 1.177.2.3  2005/02/13 23:44:04  csoutheren
- * Backported more IPV6 fixes from Atlas-devel
+ * Revision 1.197.2.2  2006/02/13 07:11:23  csoutheren
+ * Backported fix from CVS head
+ *
+ * Revision 1.197.2.1  2006/02/11 12:09:33  csoutheren
+ * Backport from CVS head
+ *
+ * Revision 1.201  2006/02/13 06:56:26  csoutheren
+ * Fixed problem on Windows
+ *
+ * Revision 1.200  2006/02/10 23:56:58  csoutheren
+ * Fixed compile problems on Debian and Windows
+ *
+ * Revision 1.199  2006/02/10 22:50:11  csoutheren
+ * Fixed error in last commit
+ *
+ * Revision 1.198  2006/02/10 22:47:01  csoutheren
+ * Ensure IPV6 addresses are not returned by IPSocket::GetHostAddress when
+ * IPV4 has been forced using PIPSocket::SetDefaultIpAddressFamilyV4
+ * This is intended to address the following Ekiga bug:
+ * Bug 330388 . Cannot make calls to host with IPv6 address
+ *
+ * Revision 1.197  2005/11/21 11:49:36  shorne
+ * Changed disableQos to disableGQoS to better reflect what it does
+ *
+ * Revision 1.196  2005/10/21 06:01:30  csoutheren
+ * Fixed warning on VS.NET 2005
+ *
+ * Revision 1.195  2005/09/25 10:51:23  dominance
+ * almost complete the mingw support. We'll be there soon. ;)
+ *
+ * Revision 1.194  2005/08/18 06:24:30  shorne
+ * Reversed Last Patch
+ *
+ * Revision 1.193  2005/08/18 03:42:53  shorne
+ * Chaeck for RSVP QoS Thx Zdenek
+ *
+ * Revision 1.192  2005/08/09 12:46:00  rjongbloed
+ * Fixed some platforms where in6addr_any does not exist.
  *
  * Revision 1.191  2005/07/17 09:25:40  csoutheren
  * Fixed problem in IPV6 variant of PIPSocket::Address::IsLoopback
  * Thanks to Roger Hardiman
  *
+ * Revision 1.190  2005/07/13 12:08:09  csoutheren
+ * Fixed QoS patches to be more consistent with PWLib style and to allow Unix compatibility
+ *
+ * Revision 1.189  2005/07/13 11:48:54  csoutheren
+ * Backported QOS changes from isvo branch
+ *
  * Revision 1.188  2005/06/21 22:28:32  rjongbloed
  * Assured IP is set to zero, so if parse of dotted decimal fails is not random IP address.
  *
+ * Revision 1.187.2.1  2005/04/25 13:42:28  shorne
+ * Extended QoS support for per-call negotiation
+ *
  * Revision 1.187  2005/03/22 07:29:30  csoutheren
  * Fixed problem where PStrings sometimes get case into
  * PIPSocket::Address when outputting to an ostream
@@ -52,15 +97,9 @@
  * Expanded interface list routines to include IPV6 addresses
  * Added IPV6 to GetLocalAddress
  *
- * Revision 1.177.2.2  2005/02/07 00:54:33  csoutheren
- * Backported latest IPV6 changes from Atlas-devel
- *
  * Revision 1.184  2005/02/07 00:47:18  csoutheren
  * Changed IPV6 code to use standard IPV6 macros
  *
- * Revision 1.177.2.1  2005/02/04 05:19:10  csoutheren
- * Backported patches from Atlas-devel
- *
  * Revision 1.183  2005/01/26 05:38:01  csoutheren
  * Added ability to remove config file support
  *
@@ -667,35 +706,20 @@ extern "C" void inet_ntoa_b(struct in_addr inetAddress, char *pString);
 
 #ifndef _WIN32_WCE
 
-class PWinQoS : public PObject
-{
-    PCLASSINFO(PWinQoS,PObject);
-
-public:
-    PWinQoS(PQoS & pqos, struct sockaddr * to, char * inBuf, DWORD & bufLen);
-    ~PWinQoS();
-    
-    //QOS qos;
-    //QOS_DESTADDR qosdestaddr;
-protected:
-    sockaddr * sa;
-};
-
-
 void CALLBACK CompletionRoutine(DWORD dwError,
                                 DWORD cbTransferred,
                                 LPWSAOVERLAPPED lpOverlapped,
                                 DWORD dwFlags);
                                 
 
-#endif	// _WIN32_WCE
-#endif	// _WIN32
+#endif  // _WIN32_WCE
+#endif  // _WIN32
 #endif // P_HAS_QOS
 
 ///////////////////////////////////////////////////////////////////////////////
 // PIPSocket::Address
 
-static int defaultIpAddressFamily=PF_INET; //PF_INET for IPv4 (default) or PF_INET6 for IPv6
+static int defaultIpAddressFamily = PF_INET;  // PF_UNSPEC;   // default to IPV4
 
 static PIPSocket::Address loopback4(127,0,0,1);
 static PIPSocket::Address broadcast4(INADDR_BROADCAST);
@@ -703,7 +727,7 @@ static PIPSocket::Address any4(INADDR_ANY);
 static in_addr inaddr_empty;
 #if P_HAS_IPV6
 static PIPSocket::Address loopback6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\001");
-static PIPSocket::Address any6(in6addr_any); 
+static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"); 
 #endif
 
 
@@ -749,7 +773,7 @@ BOOL PIPSocket::IsIpAddressFamilyV6Supported()
 PIPSocket::Address PIPSocket::GetDefaultIpAny()
 {
 #if P_HAS_IPV6
-  if (defaultIpAddressFamily == PF_INET6)
+  if (defaultIpAddressFamily != PF_INET)
     return any6;
 #endif
 
@@ -883,9 +907,10 @@ class PHostByName : PHostByName_private
   friend void PIPSocket::ClearNameCache();
 };
 
-
+PMutex creationMutex;
 static PHostByName & pHostByName()
 {
+  PWaitAndSignal m(creationMutex);
   static PHostByName t;
   return t;
 }
@@ -923,6 +948,7 @@ class PHostByAddr : PHostByAddr_private
 
 static PHostByAddr & pHostByAddr()
 {
+  PWaitAndSignal m(creationMutex);
   static PHostByAddr t;
   return t;
 }
@@ -1117,6 +1143,8 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
 #if P_HAS_IPV6
     struct addrinfo *res;
     struct addrinfo hints = { AI_CANONNAME, PF_UNSPEC };
+    hints.ai_family = defaultIpAddressFamily;
+
     localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
     mutex.Wait();
 
@@ -1142,7 +1170,7 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
       localErrNo = h_errno;
     } while (localErrNo == TRY_AGAIN && --retry > 0);
 
-#elif defined(P_RTEMS) || defined(P_CYGWIN)
+#elif defined(P_RTEMS) || defined(P_CYGWIN) || defined(P_MINGW)
 
     host_info = ::gethostbyname(name);
     localErrNo = h_errno;
@@ -1162,7 +1190,7 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
                             &hostEnt,
                             buffer, REENTRANT_BUFFER_LEN,
                             &host_info,
-      		            &localErrNo) == 0)
+                            &localErrNo) == 0)
         localErrNo = NETDB_SUCCESS;
     } while (localErrNo == TRY_AGAIN && --retry > 0);
 
@@ -1275,7 +1303,7 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
       localErrNo = h_errno;
     } while (localErrNo == TRY_AGAIN && --retry > 0);
 
-#elif defined P_RTEMS || defined P_CYGWIN
+#elif defined P_RTEMS || defined P_CYGWIN || defined P_MINGW
 
     host_info = ::gethostbyaddr(addr.GetPointer(), addr.GetSize(), PF_INET);
     localErrNo = h_errno;
@@ -1316,7 +1344,7 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
     localErrNo = h_errno;
 
 #if defined(_WIN32) || defined(WINDOWS)  // Kludge to avoid strange 95 bug
-    extern P_IsOldWin95();
+    extern int P_IsOldWin95();
     if (P_IsOldWin95() && host_info != NULL && host_info->h_addr_list[0] != NULL)
       host_info->h_addr_list[1] = NULL;
 #endif
@@ -2892,218 +2920,210 @@ PUDPSocket::PUDPSocket(const PString & address, const PString & service)
   Connect(address);
 }
 
+
 BOOL PUDPSocket::ModifyQoSSpec(PQoS * qos)
 {
-    if (qos==NULL)
-        return FALSE;
+  if (qos==NULL)
+    return FALSE;
 
-    qosSpec = *qos;
-    return TRUE;
+  qosSpec = *qos;
+  return TRUE;
 }
 
+#if P_HAS_QOS
+PQoS & PUDPSocket::GetQoSSpec()
+{
+  return qosSpec;
+}
+#endif
 
 BOOL PUDPSocket::ApplyQoS()
 {
 #ifdef _WIN32_WCE
-    return FALSE;   //QoS not supported
+  return FALSE;   //QoS not supported
 #endif
 
-    char DSCPval = 0;
-    if (qosSpec.GetDSCP() < 0 ||
-        qosSpec.GetDSCP() > 63)
-    {
-        if (qosSpec.GetServiceType() == SERVICETYPE_PNOTDEFINED)
-            return TRUE;
-        else
-        {
-            switch (qosSpec.GetServiceType())
-            {
-            case SERVICETYPE_GUARANTEED:
-                DSCPval = PQoS::guaranteedDSCP;
-                break;
-            case SERVICETYPE_CONTROLLEDLOAD:
-                DSCPval = PQoS::controlledLoadDSCP;
-                break;
-            case SERVICETYPE_BESTEFFORT:
-            default:
-                DSCPval = PQoS::bestEffortDSCP;
-                break;
-            }
-        }
+  char DSCPval = 0;
+  if (qosSpec.GetDSCP() < 0 ||
+      qosSpec.GetDSCP() > 63) {
+    if (qosSpec.GetServiceType() == SERVICETYPE_PNOTDEFINED)
+      return TRUE;
+    else {
+      switch (qosSpec.GetServiceType()) {
+        case SERVICETYPE_GUARANTEED:
+          DSCPval = PQoS::guaranteedDSCP;
+          break;
+        case SERVICETYPE_CONTROLLEDLOAD:
+          DSCPval = PQoS::controlledLoadDSCP;
+          break;
+        case SERVICETYPE_BESTEFFORT:
+        default:
+          DSCPval = PQoS::bestEffortDSCP;
+          break;
+      }
     }
-    else
-        DSCPval = (char)qosSpec.GetDSCP();
+  }
+  else
+    DSCPval = (char)qosSpec.GetDSCP();
 
 #ifdef _WIN32
 #if P_HAS_QOS
+  if (disableGQoS)
+    return FALSE;
+
+  BOOL usesetsockopt = FALSE;
+
+  OSVERSIONINFO versInfo;
+  ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
+  versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+  if (!(GetVersionEx(&versInfo)))
+    usesetsockopt = TRUE;
+  else {
+    if (versInfo.dwMajorVersion < 5)
+      usesetsockopt = TRUE;
+
+    if (disableGQoS)
+          return FALSE;
 
     BOOL usesetsockopt = FALSE;
 
-    OSVERSIONINFO versInfo;
-    ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
-    versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    if (!(GetVersionEx(&versInfo)))
-        usesetsockopt = TRUE;
-    else
-    {
-        if (versInfo.dwMajorVersion < 5)
-            usesetsockopt = TRUE;
+    if (versInfo.dwMajorVersion == 5 &&
+        versInfo.dwMinorVersion == 0)
+      usesetsockopt = TRUE;         //Windows 2000 does not always support QOS_DESTADDR
+  }
 
-        if (versInfo.dwMajorVersion == 5 &&
-            versInfo.dwMinorVersion == 0)
-            usesetsockopt = TRUE;         //Windows 2000 does not always support QOS_DESTADDR
-    }
+  BOOL retval = FALSE;
+  if (!usesetsockopt && sendAddress.IsValid() && sendPort != 0) {
+    sockaddr_in sa;
+    sa.sin_family = AF_INET;
+    sa.sin_port = htons(sendPort);
+    sa.sin_addr = sendAddress;
+    memset(sa.sin_zero,0,8);
 
-    BOOL retval = FALSE;
-    if (!usesetsockopt && sendAddress.IsValid() && sendPort != 0)
-    {
-        sockaddr_in sa;
-        sa.sin_family = AF_INET;
-        sa.sin_port = htons(sendPort);
-        sa.sin_addr = sendAddress;
-        memset(sa.sin_zero,0,8);
+    char * inBuf = new char[2048];
+    memset(inBuf,0,2048);
+    DWORD bufLen = 0;
+    PWinQoS wqos(qosSpec, (struct sockaddr *)(&sa), inBuf, bufLen);
 
-        char * inBuf = new char[2048];
-        memset(inBuf,0,2048);
-        DWORD bufLen = 0;
-        PWinQoS wqos(qosSpec, (struct sockaddr *)(&sa), inBuf, bufLen);
+    DWORD dummy = 0;
+    int irval = WSAIoctl(os_handle, SIO_SET_QOS, inBuf, bufLen, NULL, 0, &dummy, NULL, NULL);
 
-        DWORD dummy = 0;
-        int irval = WSAIoctl(os_handle,
-                             SIO_SET_QOS,
-                             inBuf,
-                             bufLen,
-                             NULL,
-                             0,
-                             &dummy,
-                             NULL,
-                             NULL);
+    delete[] inBuf;
 
-        delete[] inBuf;
+    return irval == 0;
+  }
 
-        if (irval != 0)
-            return FALSE;
-        
-        return TRUE;
-    }
+  if (!usesetsockopt)
+    return retval;
 
-    if (!usesetsockopt)
-        return retval;
+#endif  // P_HAS_QOS
+#endif  // _WIN32
 
-#endif	// P_HAS_QOS
-#endif	// _WIN32
+  unsigned int setDSCP = DSCPval<<2;
 
-    unsigned int setDSCP = DSCPval<<2;
-
-    int rv = 0;
-    unsigned int curval = 0;
-    socklen_t cursize = sizeof(curval);
-    rv = ::getsockopt(os_handle,IPPROTO_IP, IP_TOS, (char *)(&curval), &cursize);
-    if (curval == setDSCP)
-        return TRUE;    //Required DSCP already set
+  int rv = 0;
+  unsigned int curval = 0;
+  socklen_t cursize = sizeof(curval);
+  rv = ::getsockopt(os_handle,IPPROTO_IP, IP_TOS, (char *)(&curval), &cursize);
+  if (curval == setDSCP)
+    return TRUE;    //Required DSCP already set
 
 
-    rv = ::setsockopt(os_handle, IPPROTO_IP, IP_TOS,
-                                     (char *)&setDSCP, sizeof(setDSCP));
+  rv = ::setsockopt(os_handle, IPPROTO_IP, IP_TOS, (char *)&setDSCP, sizeof(setDSCP));
 
-    if (rv != 0)
-    {
-        int err;
+  if (rv != 0) {
+    int err;
 #ifdef _WIN32
-	err = WSAGetLastError();
+    err = WSAGetLastError();
 #else
-	err = errno;
+    err = errno;
 #endif
-        PTRACE(3,"QOS\tsetsockopt failed with code " << err);
-        return FALSE;
-    }
+    PTRACE(3,"QOS\tsetsockopt failed with code " << err);
+    return FALSE;
+  }
     
-    return TRUE;
+  return TRUE;
 }
 
 BOOL PUDPSocket::OpenSocketGQOS(int af, int type, int proto)
 {
 #ifdef _WIN32_WCE   //QOS not supported
-    return ConvertOSError(os_handle = os_socket(af, type, proto));
+  return ConvertOSError(os_handle = os_socket(af, type, proto));
 #endif
 
 #if defined(_WIN32) && defined(P_HAS_QOS)
     
-    DWORD bufferSize = 0;
-    DWORD numProtocols, i;
-    LPWSAPROTOCOL_INFO installedProtocols, qosProtocol;
+  DWORD bufferSize = 0;
+  DWORD numProtocols, i;
+  LPWSAPROTOCOL_INFO installedProtocols, qosProtocol;
 
-    //Try to find a QOS-enabled protocol
+  //Try to find a QOS-enabled protocol
  
-    BOOL retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
+  BOOL retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
                                                             NULL,
                                                             &bufferSize));
     
-    if (numProtocols == SOCKET_ERROR &&
-        WSAGetLastError()!=WSAENOBUFS)
-    {
-        return retval;
-    }
+  if (numProtocols == SOCKET_ERROR && WSAGetLastError()!=WSAENOBUFS) 
+    return retval;
 
-    installedProtocols = (LPWSAPROTOCOL_INFO)(new BYTE[bufferSize]);
-    retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
+  installedProtocols = (LPWSAPROTOCOL_INFO)(new BYTE[bufferSize]);
+  retval = ConvertOSError(numProtocols = WSAEnumProtocols(((proto==0) ? NULL : &proto),
                                                             installedProtocols,
                                                             &bufferSize));
-    if (numProtocols == SOCKET_ERROR)
-    {
-        delete[] installedProtocols;
-        return retval;
-    }
-
-    qosProtocol = installedProtocols;
-    BOOL haveQoSproto = FALSE;
-
-
-    for (i=0; i<numProtocols; qosProtocol++, i++)
-    {
-        if ((qosProtocol->dwServiceFlags1 & XP1_QOS_SUPPORTED) &&
-            (qosProtocol->iSocketType == type) &&
-            (qosProtocol->iAddressFamily == af))
-        {
-            haveQoSproto = TRUE;
-            break;
-        }
-    }
-
-    if (haveQoSproto)
-    {
-        retval =  ConvertOSError(os_handle = WSASocket(af,
-                                                       type,
-                                                       proto,
-                                                       qosProtocol,
-                                                       0,
-                                                       WSA_FLAG_OVERLAPPED));
-    }
-    else
-    {    
-        retval = ConvertOSError(os_handle = WSASocket (af,
-                                                       type,
-                                                       proto,
-                                                       NULL,
-                                                       0,
-                                                       WSA_FLAG_OVERLAPPED));
-    }
-
+  if (numProtocols == SOCKET_ERROR) {
     delete[] installedProtocols;
-
-    if (os_handle == INVALID_SOCKET)
-        return retval;
-#else
-    BOOL retval = ConvertOSError(os_handle = os_socket(af, type, proto));
-#endif
     return retval;
+  }
+
+  qosProtocol = installedProtocols;
+  BOOL haveQoSproto = FALSE;
+
+  for (i=0; i<numProtocols; qosProtocol++, i++) {
+    if ((qosProtocol->dwServiceFlags1 & XP1_QOS_SUPPORTED) &&
+        (qosProtocol->iSocketType == type) &&
+        (qosProtocol->iAddressFamily == af)) {
+      haveQoSproto = TRUE;
+      break;
+    }
+  }
+
+  if (haveQoSproto) {
+    retval =  ConvertOSError(os_handle = WSASocket(af,
+                                                   type,
+                                                   proto,
+                                                   qosProtocol,
+                                                   0,
+                                                   WSA_FLAG_OVERLAPPED));
+  }
+  else
+  {    
+    retval = ConvertOSError(os_handle = WSASocket (af,
+                                                   type,
+                                                   proto,
+                                                   NULL,
+                                                   0,
+                                                   WSA_FLAG_OVERLAPPED));
+  }
+
+  delete[] installedProtocols;
+
+  if (os_handle == INVALID_SOCKET)
+    return retval;
+#else
+  BOOL retval = ConvertOSError(os_handle = os_socket(af, type, proto));
+#endif
+
+  return retval;
 }
 
-BOOL PUDPSocket::OpenSocket()
-{
 #ifdef _WIN32
 #ifndef _WIN32_WCE
 #ifdef P_HAS_QOS
+
+#define COULD_HAVE_QOS
+
+static BOOL CheckOSVersion()
+{
     OSVERSIONINFO versInfo;
     ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
     versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
@@ -3112,34 +3132,33 @@ BOOL PUDPSocket::OpenSocket()
         if (versInfo.dwMajorVersion > 5 ||
            (versInfo.dwMajorVersion == 5 &&
             versInfo.dwMinorVersion > 0))
-        return OpenSocketGQOS(AF_INET, SOCK_DGRAM, 0);
+          return TRUE;
     }
+  return FALSE;
+}
+
 #endif
 #endif
 #endif
 
-    return ConvertOSError(os_handle = os_socket(AF_INET,SOCK_DGRAM, 0));
+BOOL PUDPSocket::OpenSocket()
+{
+#ifdef COULD_HAVE_QOS
+  if (CheckOSVersion()) 
+    return OpenSocketGQOS(AF_INET, SOCK_DGRAM, 0);
+#endif
+
+  return ConvertOSError(os_handle = os_socket(AF_INET,SOCK_DGRAM, 0));
 }
 
 BOOL PUDPSocket::OpenSocket(int ipAdressFamily)
 {
-#ifdef _WIN32
-#ifndef _WIN32_WCE
-#ifdef P_HAS_QOS
-    OSVERSIONINFO versInfo;
-    ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
-    versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    if (GetVersionEx(&versInfo))
-    {
-        if (versInfo.dwMajorVersion > 5 ||
-           (versInfo.dwMajorVersion == 5 &&
-            versInfo.dwMinorVersion > 0))
-        return OpenSocketGQOS(ipAdressFamily, SOCK_DGRAM, 0);
-    }
+#ifdef COULD_HAVE_QOS
+  if (CheckOSVersion()) 
+    return OpenSocketGQOS(ipAdressFamily, SOCK_DGRAM, 0);
 #endif
-#endif
-#endif
-    return ConvertOSError(os_handle = os_socket(ipAdressFamily,SOCK_DGRAM, 0));
+
+  return ConvertOSError(os_handle = os_socket(ipAdressFamily,SOCK_DGRAM, 0));
 }
 
 const char * PUDPSocket::GetProtocolName() const
@@ -3198,63 +3217,4 @@ BOOL PICMPSocket::OpenSocket(int)
   return FALSE;
 }
 
-//////////////////////////////////////////////////////////////////////////////
-
-#if P_HAS_QOS
-
-#ifdef _WIN32
-#ifndef _WIN32_WCE
-
-PWinQoS::~PWinQoS()
-{
-    delete sa;
-}
-
-PWinQoS::PWinQoS(PQoS & pqos, struct sockaddr * to, char * inBuf, DWORD & bufLen)
-{
-    QOS * qos = (QOS *)inBuf;
-    
-    if (pqos.GetTokenRate() == QOS_NOT_SPECIFIED)
-        qos->SendingFlowspec.ServiceType = SERVICETYPE_BESTEFFORT;
-    else
-        qos->SendingFlowspec.ServiceType = pqos.GetServiceType();
-    
-    qos->SendingFlowspec.TokenRate = pqos.GetTokenRate();
-    qos->SendingFlowspec.TokenBucketSize = pqos.GetTokenBucketSize();
-    qos->SendingFlowspec.PeakBandwidth = pqos.GetPeakBandwidth();
-    qos->SendingFlowspec.Latency = QOS_NOT_SPECIFIED;
-    qos->SendingFlowspec.DelayVariation = QOS_NOT_SPECIFIED;
-    qos->SendingFlowspec.MaxSduSize = QOS_NOT_SPECIFIED;
-    qos->SendingFlowspec.MinimumPolicedSize = QOS_NOT_SPECIFIED;
-
-    qos->ReceivingFlowspec.ServiceType = SERVICETYPE_BESTEFFORT|SERVICE_NO_QOS_SIGNALING;
-    qos->ReceivingFlowspec.TokenRate = QOS_NOT_SPECIFIED;
-    qos->ReceivingFlowspec.TokenBucketSize = QOS_NOT_SPECIFIED;
-    qos->ReceivingFlowspec.PeakBandwidth = QOS_NOT_SPECIFIED;
-    qos->ReceivingFlowspec.Latency = QOS_NOT_SPECIFIED;
-    qos->ReceivingFlowspec.DelayVariation = QOS_NOT_SPECIFIED;
-    qos->ReceivingFlowspec.MaxSduSize = QOS_NOT_SPECIFIED;
-    qos->ReceivingFlowspec.MinimumPolicedSize = QOS_NOT_SPECIFIED;
-
-    sa = new sockaddr;
-    *sa = *to;
-
-    QOS_DESTADDR qosdestaddr;
-    qosdestaddr.ObjectHdr.ObjectType = QOS_OBJECT_DESTADDR;
-    qosdestaddr.ObjectHdr.ObjectLength = sizeof(qosdestaddr);
-    qosdestaddr.SocketAddress = sa;
-    qosdestaddr.SocketAddressLength = sizeof(*sa);
-
-    qos->ProviderSpecific.len = sizeof(qosdestaddr);
-    qos->ProviderSpecific.buf = inBuf + sizeof(*qos);
-
-    memcpy(inBuf+sizeof(*qos),&qosdestaddr,sizeof(qosdestaddr));
-    bufLen = sizeof(*qos)+sizeof(qosdestaddr);
-}
-
-#endif // _WIN32
-#endif // _WIN32_WCE
-
-#endif // P_HAS_QOS
-
 // End Of File ///////////////////////////////////////////////////////////////
